class Network:
    BSC = "bsc"
    ETH = "eth"
    ATC = "atc"
    MATIC = "matic"
    TRX = "trx"
    XBN = "xbn"
    BASE = "base"
    
class AssetType: 
    FIAT = "fiat"
    WRAPPED = "wrapped" 
    ENAIRA = "enaira"



class ProviderType:
    KORAPAY = "korapay"
    BUDPAY = "budpay"
    # Add other provider types as needed
